<?php

namespace App\Console;

use App\Service\BackupService;
use App\Service\DatabaseService;
use App\Service\Folders;
use App\Service\Traits\SelectDatabaseQuestion;
use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

#[AsCommand('app:db:backup')]
class BackupDatabaseCommand extends Command
{
    use SelectDatabaseQuestion;

    public function __construct(
        private readonly BackupService   $bs,
        private readonly DatabaseService $db,
        private readonly Folders         $folders,
    )
    {
        parent::__construct();
    }

    protected function configure(): void
    {
        $this->addArgument('database', InputArgument::OPTIONAL, 'Database name');
        $this->addArgument('file', InputArgument::OPTIONAL, 'Backup file name');
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $db = $this->getDatabaseName($input, $output);
        $fileName = $input->getArgument('file');

        if (!$fileName) {
            $date = date('y_m_d_H_i');
            $fileName = sprintf('%s_%s_backup.sql', $db, $date);
        }

        $output->writeln(sprintf('Start backup of %s to %s', $db, $fileName));;
        $backupName = $this->bs->backup($db, $this->folders->getBackupFolder() . DIRECTORY_SEPARATOR . $fileName);
        $output->writeln(sprintf('Backup written to: %s', $backupName));

        return Command::SUCCESS;
    }
}