<?php

namespace App\Console;

use App\Service\BackupService;
use App\Service\DatabaseService;
use App\Service\Folders;
use App\Service\Traits\SelectDatabaseQuestion;
use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Question\ChoiceQuestion;
use Symfony\Component\Finder\Finder;

#[AsCommand('app:db:restore')]
class RestoreDatabaseCommand extends Command
{
    use SelectDatabaseQuestion;

    public function __construct(
        private readonly DatabaseService $db,
        private readonly BackupService   $bs,
        private readonly Folders         $folders,
    )
    {
        parent::__construct();
    }

    protected function configure(): void
    {
        $this->addArgument('database', InputArgument::OPTIONAL, 'Database name');
        $this->addArgument('file', InputArgument::OPTIONAL, 'Backup file name');
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $db = $this->getDatabaseName($input, $output);
        $fileName = $input->getArgument('file');
        if (!$fileName) {
            $fileName = $this->getBackupFileName($input, $output, $db);
        }

        $output->writeln(sprintf('Start restore of %s from %s', $db, $fileName));;
        $this->bs->restore($db, $this->folders->getBackupFolder() . DIRECTORY_SEPARATOR . $fileName);
        $output->writeln(sprintf('Restored from: %s', $fileName));

        return Command::SUCCESS;
    }

    private function getBackupFileName(InputInterface $input, OutputInterface $output, string $db): string
    {
        $finder = new Finder();
        $finder->in($this->folders->getBackupFolder());
        $files = [];
        foreach ($finder->files()->name(sprintf('%s_*_backup.sql', $db)) as $file) {
            $files[] = $file->getFilename();
        }
        rsort($files);
        $selectQuestion = new ChoiceQuestion(sprintf('Backup name (%s_*_backup.sql)', $db), $files, 0);
        $question = $this->getHelper('question');
        return $question->ask($input, $output, $selectQuestion);
    }
}